import logging

from kyoka.callback import BaseCallback, BasePerformanceWatcher
from examples.maze.helper import measure_performance, visualize_policy

class MazePerformanceWatcher(BasePerformanceWatcher):

    def define_performance_test_interval(self):
        return 1

    def run_performance_test(self, task, value_function):
        step_to_goal = measure_performance(task, value_function)
        policy = visualize_policy(task, value_function)
        return step_to_goal, policy

    def define_log_message(self, iteration_count, task, value_function, test_result):
        step_to_goal, _ = test_result
        return "Step = %d (nb_iteration=%d)" % (step_to_goal,iteration_count)

    def tearDown(self, task, value_function):
        msg_prefix = "Policy which agent learned is like this.\n"
        self.log(msg_prefix + visualize_policy(task, value_function))

class MazeTransformer(BaseCallback):

    def __init__(self):
        BaseCallback.__init__(self)
        self.transformation = {}

    def set_transformation(self, timing, maze_path):
        self.transformation[timing] = maze_path

    def before_update(self, iteration_count, task, value_function):
        if iteration_count in self.transformation:
            maze_filepath = self.transformation[iteration_count]
            task.read_maze(maze_filepath)
            logging.debug("Maze transformed into [ %s ]" % maze_filepath)




import os

from kyoka.utils import value_function_check
from kyoka.policy import GreedyPolicy
from kyoka.value_function import BaseTabularActionValueFunction, BaseApproxActionValueFunction
from kyoka.algorithm.rl_algorithm import BaseRLAlgorithm

class QLearning(BaseRLAlgorithm):

    def __init__(self, alpha=0.1, gamma=0.9):
        self.alpha = alpha
        self.gamma = gamma

    def setup(self, task, policy, value_function):
        validate_value_function(value_function)
        super(QLearning, self).setup(task, policy, value_function)
        self.greedy_policy = GreedyPolicy()

    def run_gpi_for_an_episode(self, task, policy, value_function):
        state = task.generate_initial_state()
        action = policy.choose_action(task, value_function, state)
        while not task.is_terminal_state(state):
            next_state = task.transit_state(state, action)
            next_action = choose_action(task, policy, value_function, next_state)
            reward = task.calculate_reward(next_state)
            greedy_action = choose_action(task, self.greedy_policy, value_function, next_state)
            greedy_Q_value = predict_value(value_function, next_state, greedy_action)
            backup_target = reward + self.gamma * greedy_Q_value
            value_function.backup(state, action, backup_target, self.alpha)
            state, action = next_state, next_action

class QLearningTabularActionValueFunction(BaseTabularActionValueFunction):

    def define_save_file_prefix(self):
        return "q_learning"

    def backup(self, state, action, backup_target, alpha):
        Q_value = self.predict_value(state, action)
        new_Q_value = Q_value + alpha * (backup_target - Q_value)
        self.insert_value_into_table(self.table, state, action, new_Q_value)

class QLearningApproxActionValueFunction(BaseApproxActionValueFunction):
    pass

ACTION_ON_TERMINAL_FLG = "action_on_terminal"

def choose_action(task, policy, value_function, state):
    if task.is_terminal_state(state):
        return ACTION_ON_TERMINAL_FLG
    else:
        return policy.choose_action(task, value_function, state)

def predict_value(value_function, next_state, next_action):
    if ACTION_ON_TERMINAL_FLG == next_action:
        return 0
    else:
        return value_function.predict_value(next_state, next_action)

def validate_value_function(value_function):
    value_function_check("QLearning",\
            [QLearningTabularActionValueFunction, QLearningApproxActionValueFunction],\
            value_function)




from __future__ import absolute_import

import os
from kyoka.policy import BasePolicy

def generate_tmp_dir_path(script_path):
    return os.path.join(os.path.dirname(script_path), "tmp")

def setup_tmp_dir(script_path):
    os.mkdir(generate_tmp_dir_path(script_path))

def teardown_tmp_dir(script_path, file_names):
    dir_path = generate_tmp_dir_path(script_path)
    remove_leaf_dir(dir_path, file_names)

def remove_leaf_dir(dir_path, file_names):
    if os.path.exists(dir_path):
        for file_name in file_names:
            file_path = os.path.join(dir_path, file_name)
            if os.path.exists(file_path): os.remove(file_path)
        os.rmdir(dir_path)

class NegativePolicy(BasePolicy):

    def choose_action(self, task, value_function, state):
        actions = task.generate_possible_actions(state)
        Q_value_for_actions = [value_function.predict_value(state, action) for action in actions]
        min_Q_value = min(Q_value_for_actions)
        Q_act_pair = zip(Q_value_for_actions, actions)
        worst_actions = [act for Q_value, act in Q_act_pair if min_Q_value == Q_value]
        return worst_actions[0]




import math
from kyoka.task import BaseTask

class TickTackToeTask(BaseTask):

    REWARD_WIN = 1
    REWARD_LOSE = -1
    REWARD_NONE = 0

    def __init__(self, is_first_player=True):
        self.is_first_player = is_first_player

    def generate_initial_state(self):
        first_player_board = 0
        second_player_board = 0
        state = (first_player_board, second_player_board)
        return state

    def is_terminal_state(self, state):
        return any(map(is_winning, state) + [is_draw(self, state)])

    def transit_state(self, state, action):
        first_player_board ,second_player_board = state
        count_move = lambda : bin(first_player_board | second_player_board).count("1")
        if count_move()%2==0:
            first_player_board |= action
        else:
            second_player_board |= action
        next_state = (first_player_board, second_player_board)
        return next_state

    def generate_possible_actions(self, state):
        board = state[0] | state[1]
        def add(ary, pos):
            if (board >> pos)&1 == 0: ary.append(1<<pos)
            return ary
        return reduce(add, range(9), [])

    def calculate_reward(self, state):
        first_player_board, second_player_board = state
        reward = self.REWARD_NONE
        if is_winning(first_player_board):
            reward = self.REWARD_WIN
        elif is_winning(second_player_board):
            reward = self.REWARD_LOSE
        return reward if self.is_first_player else -reward


def is_winning(player_board):
    bin2i = lambda b: int(b, 2)
    line_horizon = any([player_board & mask == mask for mask in map(bin2i, ['000000111', '000111000', '111000000'])])
    line_vertical = any([player_board & mask == mask for mask in map(bin2i, ['001001001', '010010010', '100100100'])])
    line_diagonal = any([player_board & mask == mask for mask in map(bin2i, ['100010001', '001010100'])])
    return line_horizon | line_vertical | line_diagonal

def is_draw(task, state):
    return len(task.generate_possible_actions(state)) == 0




import os
from kyoka.utils import build_not_implemented_msg, pickle_data, unpickle_data

class BaseActionValueFunction(object):

    def predict_value(self, state, action):
        err_msg = build_not_implemented_msg(self, "predict_value")
        raise NotImplementedError(err_msg)

    def backup(self, state, action, backup_target, alpha):
        err_msg = build_not_implemented_msg(self, "backup")
        raise NotImplementedError(err_msg)

    def setup(self):
      pass

    def save(self, save_dir_path):
      pass

    def load(self, load_dir_path):
      pass


class BaseTabularActionValueFunction(BaseActionValueFunction):

    BASE_SAVE_FILE_NAME = "table_action_value_function_data.pickle"

    def generate_initial_table(self):
        err_msg = build_not_implemented_msg(self, "generate_initial_table")
        raise NotImplementedError(err_msg)

    def fetch_value_from_table(self, table, state, action):
        err_msg = build_not_implemented_msg(self, "fetch_value_from_table")
        raise NotImplementedError(err_msg)

    def insert_value_into_table(self, table, state, action, new_value):
        err_msg = build_not_implemented_msg(self, "insert_value_into_table")
        raise NotImplementedError(err_msg)

    def define_save_file_prefix(self):
        return ""

    def setup(self):
        self.table = self.generate_initial_table()

    def predict_value(self, state, action):
        return self.fetch_value_from_table(self.table, state, action)

    def save(self, save_dir_path):
        pickle_data(self._gen_table_data_file_path(save_dir_path), self.table)

    def load(self, load_dir_path):
        file_path = self._gen_table_data_file_path(load_dir_path)
        if not os.path.exists(file_path):
            raise IOError('The saved data of "TableActionValueFunction" is not found on [ %s ]' % load_dir_path)
        self.table = unpickle_data(file_path)

    def _gen_table_data_file_path(self, dir_path):
        return os.path.join(dir_path, self._gen_table_data_file_name())

    def _gen_table_data_file_name(self):
        prefix = self.define_save_file_prefix()
        if len(prefix) != 0: prefix += "_"
        return prefix + self.BASE_SAVE_FILE_NAME


class BaseApproxActionValueFunction(BaseActionValueFunction):

    def predict_value(self, state, action):
        return self.approx_predict_value(self.construct_features(state, action))

    def backup(self, state, action, backup_target, alpha):
        self.approx_backup(self.construct_features(state, action), backup_target, alpha)

    def construct_features(self, state, action):
        err_msg = build_not_implemented_msg(self, "construct_features")
        raise NotImplementedError(err_msg)

    def approx_predict_value(self, features):
        err_msg = build_not_implemented_msg(self, "approx_predict_value")
        raise NotImplementedError(err_msg)

    def approx_backup(self, features, backup_target, alpha):
        err_msg = build_not_implemented_msg(self, "approx_backup")
        raise NotImplementedError(err_msg)




import os
import time
from utils import build_not_implemented_msg

class BaseCallback(object):

  def before_gpi_start(self, task, value_function):
    pass

  def before_update(self, iteration_count, task, value_function):
    pass

  def after_update(self, iteration_count, task, value_function):
    pass

  def after_gpi_finish(self, task, value_function):
    pass

  def interrupt_gpi(self, iteration_count, task, value_function):
    return False

  def define_log_tag(self):
    return self.__class__.__name__

  def log(self, message):
    if message and len(message) != 0:
      print "[%s] %s" % (self.define_log_tag(), message)

class BasePerformanceWatcher(BaseCallback):

    def setUp(self, task, value_function):
        pass

    def tearDown(self, task, value_function):
        pass

    def define_performance_test_interval(self):
        err_msg = build_not_implemented_msg(self, "define_performance_test_interval")
        raise NotImplementedError(err_msg)

    def run_performance_test(self, task, value_function):
        err_msg = build_not_implemented_msg(self, "run_performance_test")
        raise NotImplementedError(err_msg)

    def define_log_message(self, iteration_count, task, value_function, test_result):
        base_msg = "Performance test result : %s (nb_iteration=%d)"
        return base_msg % (test_result, iteration_count)


    def before_gpi_start(self, task, value_function):
        self.performance_log = []
        self.test_interval = self.define_performance_test_interval()
        self.setUp(task, value_function)

    def after_update(self, iteration_count, task, value_function):
        if iteration_count % self.test_interval == 0:
            result = self.run_performance_test(task, value_function)
            self.performance_log.append(result)
            message = self.define_log_message(iteration_count, task, value_function, result)
            self.log(message)

    def after_gpi_finish(self, task, value_function):
        self.tearDown(task, value_function)

class EpsilonAnnealer(BaseCallback):

    def __init__(self, epsilon_greedy_policy):
        self.policy = epsilon_greedy_policy
        self.anneal_finished = False

    def define_log_tag(self):
        return "EpsilonGreedyAnnealing"

    def before_gpi_start(self, _task, _value_function):
        start_msg = "Anneal epsilon from %s to %s." % (self.policy.eps, self.policy.min_eps)
        self.log(start_msg)

    def after_update(self, iteration_count, _task, _value_function):
        self.policy.anneal_eps()
        if not self.anneal_finished and self.policy.eps == self.policy.min_eps:
            self.anneal_finished = True
            finish_msg = "Annealing has finished at %d iteration." % iteration_count
            self.log(finish_msg)

class LearningRecorder(BaseCallback):

    def __init__(self, algorithm, root_save_dir_path, save_interval):
        self.algorithm = algorithm
        self.root_save_dir_path = root_save_dir_path
        self.save_interval = save_interval

    def before_gpi_start(self, _task, _value_function):
        if not os.path.exists(self.root_save_dir_path):
            err_msg = "Directory [ %s ] not found which you passed to LearningRecorder."
            raise Exception(err_msg  % self.root_save_dir_path)
        base_msg = 'Your algorithm will be saved after each %d iteration on directory [ %s ].'
        self.log(base_msg % (self.save_interval, self.root_save_dir_path))

    def after_update(self, iteration_count, _task, _value_function):
        if iteration_count % self.save_interval == 0:
            dir_name = self.define_checkpoint_save_dir_name(iteration_count)
            save_path = os.path.join(self.root_save_dir_path, dir_name)
            os.mkdir(save_path)
            self.algorithm.save(save_path)
            base_msg = "Saved algorithm after %d iteration at [ %s ]."
            self.log(base_msg % (iteration_count, save_path))

    def after_gpi_finish(self, task, value_function):
        dir_name = self.define_finish_save_dir_name()
        save_path = os.path.join(self.root_save_dir_path, dir_name)
        os.mkdir(save_path)
        self.algorithm.save(save_path)

    def define_checkpoint_save_dir_name(self, iteration_count):
        return "after_%d_iteration" % iteration_count

    def define_finish_save_dir_name(self):
        return "gpi_finished"

class BaseFinishRule(BaseCallback):

    def check_condition(self, iteration_count, task, value_function):
        err_msg = build_not_implemented_msg(self, "check_condition")
        raise NotImplementedError(err_msg)

    def generate_start_message(self):
        err_msg = build_not_implemented_msg(self, "generate_start_message")
        raise NotImplementedError(err_msg)

    def generate_finish_message(self, iteration_count):
        err_msg = build_not_implemented_msg(self, "generate_finish_message")
        raise NotImplementedError(err_msg)

    def before_gpi_start(self, task, value_function):
        self.log(self.generate_start_message())

    def interrupt_gpi(self, iteration_count, task, value_function):
        finish_iteration = self.check_condition(iteration_count, task, value_function)
        if finish_iteration: self.log(self.generate_finish_message(iteration_count))
        return finish_iteration

class ManualInterruption(BaseFinishRule):

    TARGET_WARD = "stop"

    def __init__(self, monitor_file_path, watch_interval=30):
        self.monitor_file_path = monitor_file_path
        self.watch_interval = watch_interval

    def check_condition(self, _iteration_count, _task, _value_function):
        current_time = time.time()
        if current_time - self.last_check_time >= self.watch_interval:
            self.last_check_time = current_time
            return self.__order_found_in_monitoring_file(self.monitor_file_path, self.TARGET_WARD)
        else:
            return False

    def generate_start_message(self):
        self.last_check_time = time.time()
        base_first_msg ='Write word "%s" on file "%s" will finish the GPI'
        base_second_msg = "(Stopping GPI may take about %s seconds. Because we check target file every %s seconds.)"
        first_msg = base_first_msg % (self.TARGET_WARD, self.monitor_file_path)
        second_msg = base_second_msg % (self.watch_interval, self.watch_interval)
        return "\n".join([first_msg, second_msg])

    def generate_finish_message(self, iteration_count):
        base_msg = "Interrupt GPI after %d iterations because interupption order found in [ %s ]."
        return base_msg % (iteration_count, self.monitor_file_path)

    def __order_found_in_monitoring_file(self, filepath, target_word):
        return os.path.isfile(filepath) and self.__found_target_ward_in_file(filepath, target_word)

    def __found_target_ward_in_file(self, filepath, target_word):
        search_word = lambda src, target: target in src
        src = self.__read_data(filepath)
        return search_word(src, target_word) if src else False

    def __read_data(self, filepath):
        with open(filepath, 'rb') as f: return f.read()

class WatchIterationCount(BaseFinishRule):

    def __init__(self, target_count, verbose=1):
        self.target_count = target_count
        self.start_time = self.last_update_time = 0
        self.verbose = verbose

    def define_log_tag(self):
        return "Progress"

    def check_condition(self, iteration_count, task, value_function):
        return iteration_count >= self.target_count

    def generate_start_message(self):
        self.start_time = self.last_update_time = time.time()
        return "Start GPI iteration for %d times" % self.target_count

    def generate_finish_message(self, iteration_count):
        base_msg = "Completed GPI iteration for %d times. (total time: %ds)"
        return base_msg % (iteration_count, time.time() - self.start_time)

    def before_update(self, iteration_count, task, value_function):
        super(WatchIterationCount, self).before_update(iteration_count, task, value_function)
        self.last_update_time = time.time()

    def after_update(self, iteration_count, task, value_function):
        super(WatchIterationCount, self).after_update(iteration_count, task, value_function)
        if self.verbose > 0:
            current_time = time.time()
            msg = "Finished %d / %d iterations (%.1fs)" %\
                    (iteration_count, self.target_count, current_time - self.last_update_time)
            self.last_update_time = current_time
            self.log(msg)




import os

from tests.base_unittest import BaseUnitTest
from tests.utils import generate_tmp_dir_path, setup_tmp_dir, teardown_tmp_dir, NegativePolicy
from mock import Mock
from nose.tools import raises

from kyoka.value_function import BaseActionValueFunction
from kyoka.algorithm.sarsa import Sarsa, SarsaTabularActionValueFunction,\
        SarsaApproxActionValueFunction, validate_value_function

class SarsaTest(BaseUnitTest):

    def test_value_function_validation(self):
        algo = Sarsa()
        validate_value_function(SarsaTabularActionValueFunction())
        validate_value_function(SarsaApproxActionValueFunction())
        with self.assertRaises(TypeError) as e:
            algo.setup("dummy", "dummy", BaseActionValueFunction())
        self.include("Sarsa", e.exception.message)

    def test_run_gpi_for_an_episode(self):
        algo = Sarsa(alpha=0.5, gamma=0.1)
        task = setup_stub_task()
        value_func = SarsaTabularActionValueFunctionImpl()
        policy = NegativePolicy()
        algo.setup(task, policy, value_func)

        value_func.insert_value_into_table(value_func.table, 1, 2, 10)
        value_func.insert_value_into_table(value_func.table, 1, 3, 11)
        value_func.insert_value_into_table(value_func.table, 3, 4, 100)
        value_func.insert_value_into_table(value_func.table, 3, 5, 101)

        algo.run_gpi_for_an_episode(task, policy, value_func)

        expected = [(0, 1, 1), (1, 2, 14.5), (3, 4, 74.5)]
        for state, action, value in expected:
            self.eq(value, value_func.predict_value(state, action))


class SarsaTabularActionValueFunctionTest(BaseUnitTest):

    def test_backup(self):
        func = SarsaTabularActionValueFunctionImpl()
        func.setup()
        func.backup(state=0, action=0, backup_target=2, alpha=0.5)
        self.eq(1, func.predict_value(0, 0))
        func.backup(state=0, action=0, backup_target=2, alpha=0.5)
        self.eq(1.5, func.predict_value(0, 0))

class SarsaApproxActionValueFunctionTest(BaseUnitTest):

    def setUp(self):
        self.func = SarsaApproxActionValueFunction()

    @raises(NotImplementedError)
    def test_error_msg_when_not_implement_construct_features(self):
        self.func.construct_features("dummy", "dummy")

    @raises(NotImplementedError)
    def test_error_msg_when_not_implement_approx_predict_value(self):
        self.func.approx_predict_value("dummy")

    @raises(NotImplementedError)
    def test_error_msg_when_not_implement_approx_backup(self):
        self.func.approx_backup("dummy", "dummy", "dummy")


class SarsaTabularActionValueFunctionImpl(SarsaTabularActionValueFunction):

    def generate_initial_table(self):
        return [[0 for j in range(50)] for i in range(8)]

    def fetch_value_from_table(self, table, state, action):
        return table[state][action]

    def insert_value_into_table(self, table, state, action, new_value):
        table[state][action] = new_value

def setup_stub_task():
    mock_task = Mock()
    mock_task.generate_initial_state.return_value = 0
    mock_task.is_terminal_state.side_effect = lambda state: state == 7
    mock_task.transit_state.side_effect = lambda state, action: state + action
    mock_task.generate_possible_actions.side_effect = lambda state: [] if state == 7 else [state + 1, state + 2]
    mock_task.calculate_reward.side_effect = lambda state: state**2
    return mock_task




from kyoka.utils import build_not_implemented_msg
from kyoka.policy import EpsilonGreedyPolicy
from kyoka.callback import EpsilonAnnealer, WatchIterationCount

def generate_episode(task, policy, value_function):
    state = task.generate_initial_state()
    episode = []
    while not task.is_terminal_state(state):
        action = policy.choose_action(task, value_function, state)
        next_state = task.transit_state(state, action)
        reward = task.calculate_reward(next_state)
        episode.append((state, action, next_state, reward))
        state = next_state
    return episode

class BaseRLAlgorithm(object):

    def setup(self, task, policy, value_function):
        self.task = task
        self.value_function = value_function
        self.value_function.setup()
        self.policy = policy

    def save(self, save_dir_path):
        self.value_function.save(save_dir_path)
        self.save_algorithm_state(save_dir_path)

    def load(self, load_dir_path):
        self.value_function.load(load_dir_path)
        self.load_algorithm_state(load_dir_path)

    def save_algorithm_state(self, save_dir_path):
        pass

    def load_algorithm_state(self, load_dir_path):
        pass

    def run_gpi_for_an_episode(self, task, policy, value_function):
        err_msg = build_not_implemented_msg(self, "run_gpi_for_an_episode")
        raise NotImplementedError(err_msg)

    def run_gpi(self, nb_iteration, callbacks=None, verbose=1):
        self.__check_setup_call()
        default_finish_rule = WatchIterationCount(nb_iteration, verbose)
        callbacks = self.__setup_callbacks(default_finish_rule, callbacks)
        [callback.before_gpi_start(self.task, self.value_function) for callback in callbacks]

        iteration_counter = 1
        while True:
            [callback.before_update(iteration_counter, self.task, self.value_function) for callback in callbacks]
            self.run_gpi_for_an_episode(self.task, self.policy, self.value_function)
            [callback.after_update(iteration_counter, self.task, self.value_function) for callback in callbacks]
            for finish_rule in callbacks:
                if finish_rule.interrupt_gpi(iteration_counter, self.task, self.value_function):
                    [callback.after_gpi_finish(self.task, self.value_function) for callback in callbacks]
                    if finish_rule != default_finish_rule:
                        default_finish_rule.log(default_finish_rule.generate_finish_message(iteration_counter))
                    return
            iteration_counter += 1


    def __check_setup_call(self):
        if not all([hasattr(self, attr) for attr in ["task", "value_function", "policy"]]):
            raise Exception('You need to call "setup" method before calling "run_gpi" method.')

    def __setup_callbacks(self, default_finish_rule, user_callbacks):
        user_callbacks = self.__wrap_item_if_single(user_callbacks)
        default_callbacks = [default_finish_rule]
        if isinstance(self.policy, EpsilonGreedyPolicy) and self.policy.do_annealing:
            default_callbacks.append(EpsilonAnnealer(self.policy))
        return default_callbacks + user_callbacks

    def __wrap_item_if_single(self, item):
        if item is None: item = []
        if not isinstance(item, list): item = [item]
        return item




from utils import build_not_implemented_msg

class BaseTask(object):

  def generate_inital_state(self):
    err_msg = build_not_implemented_msg(self, "generate_inital_state")
    raise NotImplementedError(err_msg)

  def is_terminal_state(self, state):
    err_msg = build_not_implemented_msg(self, "is_terminal_state")
    raise NotImplementedError(err_msg)

  def transit_state(self, state, action):
    err_msg = build_not_implemented_msg(self, "transit_state")
    raise NotImplementedError(err_msg)

  def generate_possible_actions(self, state):
    err_msg = build_not_implemented_msg(self, "generate_possible_actions")
    raise NotImplementedError(err_msg)

  def calculate_reward(self, state):
    err_msg = build_not_implemented_msg(self, "calculate_reward")
    raise NotImplementedError(err_msg)




import unittest

class BaseUnitTest(unittest.TestCase):

  def eq(self, expected, target):
    return self.assertEqual(expected, target)

  def almosteq(self, expected, target, tolerance):
    if isinstance(expected, list):
      curry = lambda zipped: self.almosteq(zipped[0], zipped[1], tolerance)
      map(curry, zip(expected, target))
    else:
      match = expected - tolerance <= target <= expected + tolerance
      if not match:
        return self.fail("%s and %s do not match with tolerance=%f" % (expected, target, tolerance))

  def neq(self, expected, target):
    return self.assertNotEqual(expected, target)

  def true(self, target):
    return self.assertTrue(target)

  def false(self, target):
    return self.assertFalse(target)

  def include(self, target, source):
    return self.assertIn(target, source)

  def not_include(self, target, source):
    return self.assertNotIn(target, source)

  def debug(self):
    from nose.tools import set_trace; set_trace()




from kyoka.policy import GreedyPolicy

def visualize_maze(maze):
    return "\n".join(maze)

def measure_performance(task, value_function, step_limit=10000):
    policy = GreedyPolicy()
    state = task.generate_initial_state()
    step_counter = 0
    while not task.is_terminal_state(state):
        action = policy.choose_action(task, value_function, state)
        state = task.transit_state(state, action)
        step_counter += 1
        if step_counter >= step_limit: break
    return step_counter

def visualize_policy(task, value_function):
    icon_map = { task.UP: "^", task.DOWN: "v", task.RIGHT: ">", task.LEFT: "<", -1: "-", -2: "G" }
    actions = _find_best_actions_on_each_cell(task,value_function)
    flg2icon = lambda flg: icon_map[flg]
    visualized_actions = [[flg2icon(flg) for flg in line] for line in actions]
    return visualize_maze(["".join(line) for line in visualized_actions])

def construct_features(task, state, action):
    w, h = task.get_maze_shape()
    next_state = task.transit_state(state, action)
    onehot = [[1 if next_state == (row, col) else 0 for col in range(h)] for row in range(w)]
    return _flatten(onehot)

def _flatten(table):
    return [item for sublist in table for item in sublist]

def _find_best_actions_on_each_cell(task, value_function):
    height, width = task.get_maze_shape()
    goal_r, goal_c = task.generate_terminal_state()
    curry = lambda row, col: _find_single_best_action(task, value_function, row, col)
    maze_with_answer = [[curry(row, col) for col in range(width)] for row in range(height)]
    maze_with_answer[goal_r][goal_c] = -2
    return maze_with_answer

def _find_single_best_action(task, value_function, row, col):
    state = (row, col)
    actions = task.generate_possible_actions(state)
    values = [value_function.predict_value(state, action) for action in actions]
    best_actions = [act for act, val in zip(actions, values) if val == max(values)]
    if len(best_actions) == 1:
      return best_actions[0]
    else:
      return -1




import kyoka.utils as U
from tests.base_unittest import BaseUnitTest

class UtilsTest(BaseUnitTest):

    def test_build_not_implemented_msg(self):
        message = U.build_not_implemented_msg(self, "hoge")
        self.include("UtilsTest", message)
        self.include("hoge", message)

    def test_value_function_check(self):
        with self.assertRaises(TypeError) as e:
            U.value_function_check("hoge", [BaseUnitTest, UtilsTest], "value_function")
        self.include("hoge", e.exception.message)
        self.include("BaseUnitTest or UtilsTest", e.exception.message)




from kyoka.policy import BasePolicy, GreedyPolicy
from examples.ticktacktoe.task import TickTackToeTask

visualize_move = lambda flg: {-1: "-", 0:"O", 1:"X"}[flg]
fetch_line = lambda board, line_idx: board[line_idx*3:(line_idx+1)*3]
visualize_line = lambda moves: "%s %s %s" % tuple(moves)

def _bit_to_array(state):
    board = [-1 for i in range(9)]
    for i in range(2):
      for j in range(9):
        if (state[i] >> j) & 1 == 1:
          board[j] = i
    return board[::-1]


def visualize_board(state):
    board = _bit_to_array(state)
    visualized_moves = map(visualize_move, board)
    visualized_lines = [visualize_line(fetch_line(visualized_moves, i)) for i in range(3)]
    visualized_board = "\n".join(visualized_lines)
    return visualized_board

def measure_performance(task, value_function, players):
    tasks = [TickTackToeTask(is_first_player=flg) for flg in [True, False]]
    next_is_first_player = lambda state: bin(state[0]|state[1]).count("1") % 2 == 0
    next_player_idx = lambda state: 0 if next_is_first_player(state) else 1
    state = task.generate_initial_state()
    while not task.is_terminal_state(state):
        idx = next_player_idx(state)
        player_task, player = tasks[idx], players[idx]
        action = player.choose_action(player_task, value_function, state)
        state = task.transit_state(state, action)
    return task.calculate_reward(state)

def play_with_agent(value_func):
    next_is_first_player = lambda state: bin(state[0]|state[1]).count("1") % 2 == 0
    next_player_idx = lambda state: 0 if next_is_first_player(state) else 1
    def show_board(state):
        print "\n%s" % visualize_board(state)

    tasks = [TickTackToeTask(is_first_player=is_first) for is_first in [True, False]]
    players = [GreedyPolicy(), TickTackToeManualPolicy()]

    state = tasks[0].generate_initial_state()
    show_board(state)
    while not tasks[0].is_terminal_state(state):
        idx = next_player_idx(state)
        task, player = tasks[idx], players[idx]
        action = player.choose_action(task, value_func, state)
        state = task.transit_state(state, action)
        show_board(state)

def construct_features(task, state, action):
    next_state = task.transit_state(state, action)
    flg_to_ary = lambda flg: reduce(lambda acc, e: acc + [1 if (flg>>e)&1==1 else 0], range(9), [])
    multi_dim_ary = [flg_to_ary(player_board) for player_board in next_state]
    return multi_dim_ary[0] + multi_dim_ary[1]


class TickTackToeManualPolicy(BasePolicy):

    ACTION_NAME_MAP = {
            1 : "lower_right",
            2 : "lower_center",
            4 : "lower_left",
            8 : "middle_right",
            16: "middle_center",
            32: "middle_left",
            64: "upper_right",
            128: "upper_center",
            256: "upper_left"
    }

    def choose_action(self, task, value_function, state):
        message = self._ask_message(task, state) + " >> "
        action = int(raw_input(message))
        if action not in task.generate_possible_actions(state):
          return self.choose_action(task, value_function, state)
        return action

    def _ask_message(self, task, state):
        possible_actions = task.generate_possible_actions(state)
        names = [self.ACTION_NAME_MAP[action] for action in possible_actions]
        return ", ".join(["%d: %s" % info for info in zip(possible_actions, names)])




import random
import logging

from kyoka.callback import BaseCallback, BasePerformanceWatcher
from kyoka.policy import BasePolicy, GreedyPolicy
from examples.ticktacktoe.task import TickTackToeTask
from examples.ticktacktoe.helper import measure_performance

class TickTackToePerformanceWatcher(BasePerformanceWatcher):

    def __init__(self, test_interval, test_game_count, is_first_player=True):
        self.test_interval = test_interval
        self.test_game_count = test_game_count
        self.is_first_player = is_first_player

    def define_performance_test_interval(self):
        return self.test_interval

    def run_performance_test(self, task, value_function):
        players = self._setup_players(value_function)
        game_results = [measure_performance(task, value_function, players)\
                for _ in range(self.test_game_count)]
        result_count = [game_results.count(result) for result in [-1, 0, 1]]
        result_rate  = [1.0 * count / len(game_results) for count in result_count]
        return result_rate

    def define_log_message(self, iteration_count, task, value_function, test_result):
        return "VS PerfectPolicy average result: lose=%f, draw=%f, win=%f" % tuple(test_result)

    def _setup_players(self, value_function):
        tasks = [TickTackToeTask(is_first_player=is_first)\
            for is_first in [self.is_first_player, not self.is_first_player]]
        players = [GreedyPolicy(), TickTackToePerfectPolicy()]
        players = players if self.is_first_player else players[::-1]
        return players



class TickTackToePerfectPolicy(BasePolicy):

    def choose_action(self, task, value_function, state):
        actions = task.generate_possible_actions(state)
        states = [task.transit_state(state, action) for action in actions]
        values = [self.mini(task, state, -20, 20) for state in states]
        logging.debug("MiniMax calculation result [(action, score),...] => %s" % zip(actions, values))
        best_actions = [act for act, val in zip(actions, values) if val == max(values)]
        return random.choice(best_actions)


    def maxi(self, task, state, alpha, beta):
        if task.is_terminal_state(state): return task.calculate_reward(state)
        for action in task.generate_possible_actions(state):
            next_state = task.transit_state(state, action)
            score = self.mini(task, next_state, alpha, beta)
            if score >= beta:
                return beta
            if score > alpha:
                alpha = score
        return alpha

    def mini(self, task, state, alpha, beta):
        if task.is_terminal_state(state): return task.calculate_reward(state)
        best = 100
        for action in task.generate_possible_actions(state):
            next_state = task.transit_state(state, action)
            score = self.maxi(task, next_state, alpha, beta)
            if score <= alpha:
                return alpha
            if score < beta:
                beta = score
        return beta




from mock import patch
from tests.base_unittest import BaseUnitTest
from kyoka.task import BaseTask
from kyoka.algorithm.montecarlo_tree_search import BaseMCTS, BaseNode, BaseEdge,\
        UCTNode, UCTEdge, random_playout
from kyoka.callback import WatchIterationCount

class MCTSTest(BaseUnitTest):

    def setUp(self):
        self.mcts = TestMCTS(TestTask())

    def test_update_edge_value(self):
        with self.assertRaises(NotImplementedError) as e:
            BaseMCTS(TestTask()).generate_node_from_state("dummy")
        self.include("generate_node_from_state", e.exception.message)

    def test_choose_action(self):
        self.mcts.set_playout_policy(self.mcts._mock_playout)
        self.mcts.set_finish_rule(WatchIterationCount(1))
        action = self.mcts.choose_action("dummy", "dummy", "A")
        self.eq(1, action)

    def test_planning(self):
        self.mcts.set_playout_policy(self.mcts._mock_playout)

        def edge_check(edge, value, visit_count):
            self.almosteq(value, edge.value, 0.01)
            self.eq(visit_count, edge.visit_count)

        action = self.mcts.planning("A", WatchIterationCount(1, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        self.eq(1, action)
        edge_check(nodeA.child_edges[0], 2, 1)

        action = self.mcts.planning("A", WatchIterationCount(2, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        self.eq(1, action)
        edge_check(nodeA.child_edges[1], 1, 1)

        action = self.mcts.planning("A", WatchIterationCount(3, verbose=0))
        nodeB = self.mcts.last_calculated_tree.child_edges[0].child_node
        self.eq(5, action)
        edge_check(nodeB.child_edges[0], 0.5, 1)

        action = self.mcts.planning("A", WatchIterationCount(4, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        self.eq(1, action)
        edge_check(nodeA.child_edges[1], 1, 2)

        action = self.mcts.planning("A", WatchIterationCount(5, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        nodeB = nodeA.child_edges[0].child_node
        self.eq(5, action)
        edge_check(nodeA.child_edges[0], 1.33, 3)
        edge_check(nodeB.child_edges[1], 1.5, 1)

        action = self.mcts.planning("A", WatchIterationCount(6, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        self.eq(1, action)
        edge_check(nodeA.child_edges[1], 1, 3)

        action = self.mcts.planning("A", WatchIterationCount(7, verbose=1))
        nodeA = self.mcts.last_calculated_tree
        nodeB = nodeA.child_edges[0].child_node
        nodeD = nodeB.child_edges[1].child_node
        self.eq(5, action)
        edge_check(nodeD.child_edges[0], 0.1, 1)
        edge_check(nodeB.child_edges[1], 0.8, 2)
        edge_check(nodeA.child_edges[0], 1.025, 4)

        action = self.mcts.planning("A", WatchIterationCount(8, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        self.eq(1, action)
        edge_check(nodeA.child_edges[1], 1, 4)

        action = self.mcts.planning("A", WatchIterationCount(9, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        nodeB = nodeA.child_edges[0].child_node
        self.eq(5, action)
        edge_check(nodeB.child_edges[0], 0.5, 2)
        edge_check(nodeA.child_edges[0], 0.919, 5)

        action = self.mcts.planning("A", WatchIterationCount(10, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        self.eq(5, action)
        edge_check(nodeA.child_edges[1], 1, 5)

        action = self.mcts.planning("A", WatchIterationCount(11, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        self.eq(1, action)
        edge_check(nodeA.child_edges[1], 1, 6)

        action = self.mcts.planning("A", WatchIterationCount(12, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        nodeB = nodeA.child_edges[0].child_node
        nodeD = nodeB.child_edges[1].child_node
        self.eq(5, action)
        edge_check(nodeD.child_edges[0], 0.1, 2)
        edge_check(nodeB.child_edges[1], 0.566, 3)
        edge_check(nodeA.child_edges[0], 0.78, 6)

        action = self.mcts.planning("A", WatchIterationCount(13, verbose=0))
        nodeA = self.mcts.last_calculated_tree
        self.eq(1, action)
        edge_check(nodeA.child_edges[1], 1, 7)

        self.almosteq(2.94, nodeA.child_edges[0].calculate_value(), 0.01)
        self.almosteq(2.85, nodeA.child_edges[1].calculate_value(), 0.01)

    def test_select(self):
        root = self.mcts.generate_node_from_state("A")
        self.eq("A", self.mcts._select(root).state)
        root.child_edges[0].build_child(self.mcts.generate_node_from_state)
        self.eq("A", self.mcts._select(root).state)
        root.child_edges[1].build_child(self.mcts.generate_node_from_state)

        root.child_edges[1].value = 5
        self.eq("F", self.mcts._select(root).state)

        root.child_edges[0].value = 10
        self.eq("B", self.mcts._select(root).state)

        nodeB = root.child_edges[0].child_node
        nodeB.child_edges[0].build_child(self.mcts.generate_node_from_state)
        self.eq("B", self.mcts._select(root).state)
        nodeB.child_edges[1].build_child(self.mcts.generate_node_from_state)
        nodeB.child_edges[1].value = 10
        self.eq("D", self.mcts._select(root).state)

        nodeD = nodeB.child_edges[1].child_node
        nodeD.child_edges[0].build_child(self.mcts.generate_node_from_state)
        self.eq("E", self.mcts._select(root).state)

        nodeB.child_edges[0].value = 20
        self.eq("C", self.mcts._select(root).state)

    def test_expand(self):
        root = self.mcts.generate_node_from_state("A")
        self.assertIsNone(root.child_edges[0].child_node)
        self.assertIsNone(root.child_edges[1].child_node)
        self.mcts._expand(root)
        self.eq("B", root.child_edges[0].child_node.state)
        self.mcts._expand(root)
        self.eq("F", root.child_edges[1].child_node.state)
        with self.assertRaises(AssertionError) as e:
            self.mcts._expand(root)

    def test_playout(self):
        root = self.mcts.generate_node_from_state("A")
        self.mcts._expand(root)
        self.mcts._expand(root)
        nodeF = root.child_edges[1].child_node
        with patch("random.choice", side_effect=lambda ary: ary[0]):
            self.eq(0.5, self.mcts._playout(root))
        self.eq(1, self.mcts._playout(nodeF))

    def test_backpropagation(self):
        nodeA = self.mcts.generate_node_from_state("A")
        nodeA.child_edges[0].build_child(self.mcts.generate_node_from_state)
        nodeA.child_edges[1].build_child(self.mcts.generate_node_from_state)
        nodeB = nodeA.child_edges[0].child_node
        nodeB.child_edges[0].build_child(self.mcts.generate_node_from_state)
        nodeB.child_edges[1].build_child(self.mcts.generate_node_from_state)
        nodeC = nodeB.child_edges[0].child_node
        nodeD = nodeB.child_edges[1].child_node
        nodeD.child_edges[0].build_child(self.mcts.generate_node_from_state)
        nodeE = nodeD.child_edges[0].child_node
        nodeF = nodeA.child_edges[1].child_node
        edge1 = nodeA.child_edges[0]
        edge2 = nodeB.child_edges[0]
        edge3 = nodeB.child_edges[1]
        edge4 = nodeD.child_edges[0]
        edge5 = nodeA.child_edges[1]

        def subtest(edge_values, visit_counts):
            self.eq(edge1.value, edge_values[0])
            self.eq(edge1.visit_count, visit_counts[0])
            self.eq(edge2.value, edge_values[1])
            self.eq(edge2.visit_count, visit_counts[1])
            self.eq(edge3.value, edge_values[2])
            self.eq(edge3.visit_count, visit_counts[2])
            self.eq(edge4.value, edge_values[3])
            self.eq(edge4.visit_count, visit_counts[3])
            self.eq(edge5.value, edge_values[4])
            self.eq(edge5.visit_count, visit_counts[4])

        self.mcts._backpropagation(nodeB, 1)
        subtest([1, 0, 0, 0, 0], [1, 0, 0, 0, 0])
        self.mcts._backpropagation(nodeC, 3)
        subtest([2, 3, 0, 0, 0], [2, 1, 0, 0, 0])
        self.mcts._backpropagation(nodeD, 5)
        subtest([3, 3, 5, 0, 0], [3, 1, 1, 0, 0])
        self.mcts._backpropagation(nodeE, -1)
        subtest([2, 3, 2, -1, 0], [4, 1, 2, 1, 0])
        self.mcts._backpropagation(nodeF, 5)
        subtest([2, 3, 2, -1, 5], [4, 1, 2, 1, 1])

    def test_random_playout(self):
        root = self.mcts.generate_node_from_state("A")
        with patch("random.choice", side_effect=lambda ary: ary[0]):
            self.eq(0.5, random_playout(TestTask(), root))
        with patch("random.choice", side_effect=lambda ary: ary[1]):
            self.eq(1, random_playout(TestTask(), root))

class BaseNodeTest(BaseUnitTest):

    def test_generate_edge(self):
        with self.assertRaises(NotImplementedError) as e:
            BaseNode(TestTask(), "A")
        self.include("generate_edge", e.exception.message)

    def test_select_best_edge(self):
       node = TestNode(TestTask(), "A")
       node.child_edges[0].visit()
       node.child_edges[0].update_value(1)
       self.eq(1, node.select_best_edge().action)

    def test_build_child_edges(self):
       node = TestNode(TestTask(), "A")
       self.eq(2, len(node.child_edges))
       self.eq(1, node.child_edges[0].action)
       self.eq(node, node.child_edges[0].parent_node)
       self.eq(5, node.child_edges[1].action)
       self.eq(node, node.child_edges[1].parent_node)

    def test_has_unvisited_edge(self):
       node = TestNode(TestTask(), "A")
       state2node = lambda state: TestNode(TestTask(), state)
       self.true(node.has_unvisited_edge())
       node.child_edges[0].build_child(state2node)
       self.true(node.has_unvisited_edge())
       node.child_edges[1].build_child(state2node)
       self.false(node.has_unvisited_edge())

    def test_select_unvisited_edge(self):
       node = TestNode(TestTask(), "A")
       state2node = lambda state: TestNode(TestTask(), state)
       self.false(node.child_edges[0].has_child())
       node.select_unvisited_edge().build_child(state2node)
       self.true(node.child_edges[0].has_child())
       self.false(node.child_edges[1].has_child())
       node.select_unvisited_edge().build_child(state2node)
       self.true(node.child_edges[1].has_child())

    def test_visit_count(self):
       node = TestNode(TestTask(), "A")
       self.eq(0, node.visit_count())
       node.child_edges[0].visit()
       node.child_edges[0].visit()
       node.child_edges[1].visit()
       self.eq(3, node.visit_count())

class BaseEdgeTest(BaseUnitTest):

    def setUp(self):
        self.edge = BaseEdge(TestNode(TestTask(), "A"), 1)

    def test_update_value(self):
        with self.assertRaises(NotImplementedError) as e:
            self.edge.update_value("dummy")
        self.include("update_value", e.exception.message)

    def test_calculate_value(self):
        with self.assertRaises(NotImplementedError) as e:
            self.edge.calculate_value()
        self.include("calculate_value", e.exception.message)

    def test_build_child_and_had_child(self):
        edge = BaseEdge(TestNode(TestTask(), "A"), 1)
        self.false(edge.has_child())
        edge.build_child(lambda state: TestNode(TestTask(), state))
        self.eq("B", edge.child_node.state)
        self.true(edge.has_child())

    def test_visit(self):
        self.eq(0, self.edge.visit_count)
        self.edge.visit()
        self.eq(1, self.edge.visit_count)

class UCTEdgeNodeTest(BaseUnitTest):

    def setUp(self):
        self.nodeA = UCTNode(TestTask(), "A")
        self.edge = self.nodeA.child_edges[0]

    def test_update_value(self):
        self.edge.visit()
        self.edge.update_value(5)
        self.eq(5, self.edge.value)
        self.edge.visit()
        self.edge.update_value(1)
        self.eq(3, self.edge.value)

    def test_calculate_value(self):
        self.edge.visit()
        self.edge.update_value(0)
        self.almosteq(0, self.edge.calculate_value(), 0.0001)
        self.nodeA.child_edges[1].visit()
        self.almosteq(1.1774, self.edge.calculate_value(), 0.0001)
        self.edge.visit()
        self.edge.update_value(1)
        self.almosteq(1.5481, self.edge.calculate_value(), 0.0001)

class TestTask(BaseTask):

    def is_terminal_state(self, state):
        return state in ["C", "E", "F"]

    def transit_state(self, state, action):
        return { "A":{1:"B", 5:"F"}, "B":{2:"C", 3:"D"}, "D":{4:"E"} }[state][action]

    def generate_possible_actions(self, state):
        return { "A":[1,5], "B":[2,3], "C":[], "D":[4], "E":[], "F":[] }[state]

    def calculate_reward(self, state):
        return { "C": 0.5, "E": 0.1, "F": 1 }[state]

class TestMCTS(BaseMCTS):

    def generate_node_from_state(self, state):
        return TestNode(self.task, state)

    def _mock_playout(self, task, leaf_node):
        state = leaf_node.state
        if task.is_terminal_state(state):
            return task.calculate_reward(state)
        else:
            return { "B": 2, "D":1.5 }[state]

class TestNode(BaseNode):

    def generate_edge(self, parent_node, action):
        return TestEdge(parent_node, action)

class TestEdge(BaseEdge):

    def __init__(self, parent_node, action):
        super(TestEdge, self).__init__(parent_node, action)
        self.value = 0

    def update_value(self, new_reward):
        self.value = self._calc_average_in_incremental_way(self.value, self.visit_count, new_reward)

    def calculate_value(self):
        if self.visit_count == 0:
            explore_term = 0
        else:
            explore_term = 1.0 * self.parent_node.visit_count() / self.visit_count
        return self.value + explore_term

    def _calc_average_in_incremental_way(self, old_value, visit_count, new_reward):
        assert visit_count != 0
        return old_value + 1.0 / visit_count * (new_reward - old_value)




import os

from kyoka.utils import pickle_data, unpickle_data, value_function_check
from kyoka.value_function import BaseTabularActionValueFunction, BaseApproxActionValueFunction
from kyoka.algorithm.rl_algorithm import BaseRLAlgorithm, generate_episode

class MonteCarlo(BaseRLAlgorithm):

    def __init__(self, gamma=1):
        self.gamma = gamma

    def setup(self, task, policy, value_function):
        validate_value_function(value_function)
        super(MonteCarlo, self).setup(task, policy, value_function)

    def run_gpi_for_an_episode(self, task, policy, value_function):
        episode = generate_episode(task, policy, value_function)
        for idx, turn_info in enumerate(episode):
            state, action, _next_state, _reward = turn_info
            following_reward = self._calculate_following_state_reward(idx, episode)
            value_function.backup(state, action, following_reward, alpha="dummy")

    def _calculate_following_state_reward(self, current_turn, episode):
        following_turn_info = episode[current_turn:]
        following_reward = [reward for _, _, _, reward in following_turn_info]
        return sum([self.__discount(step, reward) for step, reward in enumerate(following_reward)])

    def __discount(self, step, reward):
        return self.gamma ** step * reward

class MonteCarloTabularActionValueFunction(BaseTabularActionValueFunction):

    SAVE_FILE_NAME = "montecarlo_update_counter.pickle"

    def setup(self):
        super(MonteCarloTabularActionValueFunction, self).setup()
        self.update_counter = self.generate_initial_table()

    def define_save_file_prefix(self):
        return "montecarlo"

    def save(self, save_dir_path):
        super(MonteCarloTabularActionValueFunction, self).save(save_dir_path)
        pickle_data(self._gen_update_counter_file_path(save_dir_path), self.update_counter)

    def load(self, load_dir_path):
        super(MonteCarloTabularActionValueFunction, self).load(load_dir_path)
        if not os.path.exists(self._gen_update_counter_file_path(load_dir_path)):
            raise IOError('The saved data of "MonteCarlo" algorithm is not found in [ %s ]'% load_dir_path)
        self.update_counter = unpickle_data(self._gen_update_counter_file_path(load_dir_path))

    def backup(self, state, action, backup_target, alpha):
        update_count = self.fetch_value_from_table(self.update_counter, state, action)
        Q_value = self.fetch_value_from_table(self.table, state, action)
        new_value = self._calc_average_in_incremental_way(update_count, backup_target, Q_value)
        self.insert_value_into_table(self.table, state, action, new_value)
        self.insert_value_into_table(self.update_counter, state, action, update_count+1)

    def _calc_average_in_incremental_way(self, k, r, Q):
        return Q + 1.0 / (k + 1) * (r - Q)

    def _gen_update_counter_file_path(self, dir_path):
        return os.path.join(dir_path, self.SAVE_FILE_NAME)

class MonteCarloApproxActionValueFunction(BaseApproxActionValueFunction):
    pass

def validate_value_function(value_function):
    value_function_check("MonteCarlo",\
            [MonteCarloTabularActionValueFunction, MonteCarloApproxActionValueFunction],\
            value_function)




import os
import examples.maze.helper as H

from tests.base_unittest import BaseUnitTest
from kyoka.value_function import BaseActionValueFunction
from examples.maze.task import MazeTask

class MazeHelperTest(BaseUnitTest):

    def setUp(self):
        self.task = MazeTask()
        self.task.read_maze(get_sample_maze_path())

    def test_visualize_maze(self):
        expected = \
                "-S--"+ "\n" + \
                "--XG"+ "\n" + \
                "----"
        self.eq(expected, H.visualize_maze(self.task.maze))

    def test_measure_performace(self):
        value_function = PerfectValueFunction()
        performance = H.measure_performance(self.task, value_function)
        self.eq(3, performance)

    def test_visualize_policy(self):
        value_function = PerfectValueFunction()
        expected = \
                "->>v"+ "\n" + \
                "---G"+ "\n" + \
                "----"
        self.eq(expected, H.visualize_policy(self.task, value_function))

    def test_find_best_actions_on_each_cell(self):
        value_function = PerfectValueFunction()
        answer = H._find_best_actions_on_each_cell(self.task, value_function)
        self.eq(answer[0][0], -1)
        self.eq(answer[0][1], MazeTask.RIGHT)
        self.eq(answer[0][2], MazeTask.RIGHT)
        self.eq(answer[0][3], MazeTask.DOWN)

    def test_feature_construction(self):
        maze_shape = self.task.get_maze_shape()
        state1, action1 = (0, 0), self.task.RIGHT
        self.eq([0,1,0,0,0,0,0,0,0,0,0,0], H.construct_features(self.task, state1, action1))
        state2, action2 = (2, 1), self.task.UP
        self.eq([0,0,0,0,0,1,0,0,0,0,0,0], H.construct_features(self.task, state2, action2))
        state3, action3 = self.task.generate_terminal_state(), self.task.RIGHT
        self.eq([0,0,0,0,0,0,0,1,0,0,0,0], H.construct_features(self.task, state3, action3))

class PerfectValueFunction(BaseActionValueFunction):

    CHEET_SHEET = {
        ((0,1), MazeTask.RIGHT): 1,
        ((0,2), MazeTask.RIGHT): 1,
        ((0,3), MazeTask.DOWN): 1
    }

    def predict_value(self, state, action):
        if (state, action) in self.CHEET_SHEET:
            return 1
        else:
            return 0

def get_sample_maze_path():
    return os.path.join(os.path.dirname(__file__), "sample_maze.txt")




import os
import random

from kyoka.utils import pickle_data, unpickle_data, value_function_check, build_not_implemented_msg
from kyoka.policy import GreedyPolicy, EpsilonGreedyPolicy
from kyoka.value_function import BaseApproxActionValueFunction
from kyoka.algorithm.rl_algorithm import BaseRLAlgorithm, generate_episode

class DeepQLearning(BaseRLAlgorithm):

    SAVE_FILE_NAME = "dq_replay_memory.pickle"

    def __init__(self, gamma=0.99, N=1000000, C=10000, minibatch_size=32, replay_start_size=50000):
        self.gamma = gamma
        self.replay_memory = ExperienceReplay(max_size=N)
        self.C = C
        self.minibatch_size = minibatch_size
        self.replay_start_size = replay_start_size
        self.reset_step_counter = 0

    def setup(self, task, policy, value_function):
        validate_value_function(value_function)
        super(DeepQLearning, self).setup(task, policy, value_function)
        initialize_replay_memory(task, value_function, self.replay_memory, self.replay_start_size)
        self.greedy_policy = GreedyPolicy()

    def run_gpi_for_an_episode(self, task, policy, value_function):
        value_function.use_target_network(False)
        state = task.generate_initial_state()

        while not task.is_terminal_state(state):
            action = policy.choose_action(task, value_function, state)
            next_state = task.transit_state(state, action)
            reward = task.calculate_reward(next_state)
            self.replay_memory.store_transition(state, action, reward, next_state)
            state = next_state

            experience_minibatch = self.replay_memory.sample_minibatch(self.minibatch_size)
            backup_minibatch = self._gen_backup_minibatch(task, self.greedy_policy, value_function, experience_minibatch)
            value_function.backup_on_minibatch(value_function.q_network, backup_minibatch)

            if self.reset_step_counter >= self.C:
                value_function.reset_target_network()
                self.reset_step_counter = 0
            else:
                self.reset_step_counter += 1

    def save_algorithm_state(self, save_dir_path):
        state = (
                self.gamma, self.replay_memory.dump(), self.C, self.minibatch_size,
                self.replay_start_size, self.reset_step_counter
                )
        pickle_data(self._gen_replay_memory_save_path(save_dir_path), state)

    def load_algorithm_state(self, load_dir_path):
        state = unpickle_data(self._gen_replay_memory_save_path(load_dir_path))
        self.gamma, replay_memory_serial, self.C, self.minibatch_size,\
                self.replay_start_size, self.reset_step_counter = state
        self.greedy_policy = GreedyPolicy()
        new_replay_memory = ExperienceReplay(max_size=1)
        new_replay_memory.load(replay_memory_serial)
        self.replay_memory = new_replay_memory

    def _gen_backup_minibatch(self, task, greedy_policy, value_function, experience_minibatch):
        value_function.use_target_network(True)
        backup_minibatch = [self._gen_backup_data(task, greedy_policy, value_function, experience)\
                for experience in experience_minibatch]
        value_function.use_target_network(False)
        return backup_minibatch

    def _gen_backup_data(self, task, greedy_policy, value_function, experience):
        state, action, reward, next_state = experience
        greedy_action = choose_action(task, greedy_policy, value_function, next_state)
        greedy_Q_value = predict_value(value_function, next_state, greedy_action)
        backup_target = reward + self.gamma * greedy_Q_value
        return (state, action, backup_target)

    def _gen_replay_memory_save_path(self, dir_path):
        return os.path.join(dir_path, self.SAVE_FILE_NAME)

class DeepQLearningApproxActionValueFunction(BaseApproxActionValueFunction):

    def initialize_network(self):
        err_msg = build_not_implemented_msg(self, "initialize_network")
        raise NotImplementedError(err_msg)

    def deepcopy_network(self, q_network):
        err_msg = build_not_implemented_msg(self, "deepcopy_network")
        raise NotImplementedError(err_msg)

    def predict_value_by_network(self, network, state, action):
        err_msg = build_not_implemented_msg(self, "predict_value_by_network")
        raise NotImplementedError(err_msg)

    def backup_on_minibatch(self, q_network, backup_minibatch):
        err_msg = build_not_implemented_msg(self, "backup_on_minibatch")
        raise NotImplementedError(err_msg)

    def save_networks(self, q_network, q_hat_network, save_dir_path):
        err_msg = build_not_implemented_msg(self, "save_networks")
        raise NotImplementedError(err_msg)

    def load_networks(self, load_dir_path):
        err_msg = build_not_implemented_msg(self, "load_networks")
        raise NotImplementedError(err_msg)


    def setup(self):
        self.q_network = self.initialize_network()
        self.reset_target_network()
        self.use_target_network_flg = False

    def use_target_network(self, use_target_network):
        self.use_target_network_flg = use_target_network

    def predict_value(self, state, action):
        network = self.q_hat_network if self.use_target_network_flg else self.q_network
        return self.predict_value_by_network(network, state, action)

    def reset_target_network(self):
        self.q_hat_network = self.deepcopy_network(self.q_network)

    def save(self, save_dir_path):
        self.save_networks(self.q_network, self.q_hat_network, save_dir_path)

    def load(self, load_dir_path):
        self.q_network, self.q_hat_network = self.load_networks(load_dir_path)


class ExperienceReplay(object):

    def __init__(self, max_size):
        self.max_size = max_size
        self.queue = []

    def store_transition(self, state, action, reward, next_state):
        if len(self.queue) >= self.max_size:
            self.queue.pop(0)
        self.queue.append((state, action, reward, next_state))

    def sample_minibatch(self, minibatch_size):
        return random.sample(self.queue, minibatch_size)

    def dump(self):
        return (self.max_size, self.queue)

    def load(self, serial):
        self.max_size, self.queue = serial

def initialize_replay_memory(task, value_function, replay_memory, start_size):
    random_policy = EpsilonGreedyPolicy(eps=1.0)
    while len(replay_memory.queue) < start_size:
        for state, action, next_state, reward in generate_episode(task, random_policy, value_function):
            replay_memory.store_transition(state, action, reward, next_state)
            if len(replay_memory.queue) >= start_size: return

ACTION_ON_TERMINAL_FLG = "action_on_terminal"

def choose_action(task, policy, value_function, state):
    if task.is_terminal_state(state):
        return ACTION_ON_TERMINAL_FLG
    else:
        return policy.choose_action(task, value_function, state)

def predict_value(value_function, next_state, next_action):
    if ACTION_ON_TERMINAL_FLG == next_action:
        return 0
    else:
        return value_function.predict_value(next_state, next_action)

def validate_value_function(value_function):
    value_function_check("DeepQLearning",\
            [DeepQLearningApproxActionValueFunction],\
            value_function)




from tests.base_unittest import BaseUnitTest
from kyoka.task import BaseTask
from nose.tools import raises

class BaseTaskTest(BaseUnitTest):

    def setUp(self):
        self.task = BaseTask()

    @raises(NotImplementedError)
    def test_generate_inital_state(self):
        self.task.generate_inital_state()

    @raises(NotImplementedError)
    def test_is_terminal_state(self):
        self.task.is_terminal_state("dummy")

    @raises(NotImplementedError)
    def test_transit_state(self):
        self.task.transit_state("dummy", "dummy")

    @raises(NotImplementedError)
    def test_generate_possible_actions(self):
        self.task.generate_possible_actions("dummy")

    @raises(NotImplementedError)
    def test_calculate_reward(self):
        self.task.calculate_reward("dummy")




import random
import math
from kyoka.utils import build_not_implemented_msg
from kyoka.task import BaseTask

class BaseMCTS(object):

    def __init__(self, task):
        self.task = task
        self.playout_policy = random_playout
        self.last_calculated_tree = None
        self.finish_rule = None

    def generate_node_from_state(self, state):
        err_msg = build_not_implemented_msg(self, "generate_node_from_state")
        raise NotImplementedError(err_msg)

    def set_playout_policy(self, policy):
        self.playout_policy = policy

    def set_finish_rule(self, finish_rule):
        self.finish_rule = finish_rule

    def choose_action(self, _task, _value_function, state):
        assert self.finish_rule is not None
        return self.planning(state, self.finish_rule)

    def planning(self, state, finish_rule):
        assert not self.task.is_terminal_state(state)
        finish_rule.log(finish_rule.generate_start_message().replace("GPI", "MCTS"))
        iteration_count = 0

        root_node = self.generate_node_from_state(state)  # TODO use last calculation result like AlphaGo
        while not finish_rule.check_condition(iteration_count, self.task, None):
            finish_rule.before_update(iteration_count, self.task, None)

            selected_node = self._select(root_node)
            if self.task.is_terminal_state(selected_node.state):
                leaf_node = selected_node
                reward = self.task.calculate_reward(leaf_node.state)
            else:
                leaf_node = self._expand(selected_node)
                reward = self._playout(leaf_node)
            self._backpropagation(leaf_node, reward)

            finish_rule.after_update(iteration_count, self.task, None)
            iteration_count += 1

        self.last_calculated_tree = root_node
        finish_rule.log(finish_rule.generate_finish_message(iteration_count).replace("GPI", "MCTS"))
        return root_node.select_best_edge().action

    def _select(self, root_node):
        target_node = root_node
        while not self.task.is_terminal_state(target_node.state) and not target_node.has_unvisited_edge():
            target_node = target_node.select_best_edge().child_node
        return target_node

    def _expand(self, node):
        assert node.has_unvisited_edge()
        edge = node.select_unvisited_edge()
        edge.build_child(self.generate_node_from_state)
        return edge.child_node

    def _playout(self, leaf_node):
        reward = self.playout_policy(self.task, leaf_node)
        return reward

    def _backpropagation(self, leaf_node, reward):
        target = leaf_node
        assert target.parent_edge is not None
        while target.parent_edge:
            target.parent_edge.visit()
            target.parent_edge.update_value(reward)
            target = target.parent_edge.parent_node

def random_playout(task, leaf_node, rand=random):
    state = leaf_node.state
    while not task.is_terminal_state(state):
        actions = task.generate_possible_actions(state)
        action = random.choice(actions)
        state = task.transit_state(state, action)
    return task.calculate_reward(state)


class BaseNode(object):

    def __init__(self, task, state):
        self.task = task
        self.state = state
        self.parent_edge = None
        self.child_edges = self.build_child_edges(task, state)

    def generate_edge(self, parent_node, action):
        err_msg = build_not_implemented_msg(self, "generate_edge")
        raise NotImplementedError(err_msg)

    def select_best_edge(self):
        return sorted([edge for edge in self.child_edges], key=self._edge_sort_key)[-1]

    def _edge_sort_key(self, edge):
        return edge.calculate_value()

    def build_child_edges(self, task, state):
        actions = task.generate_possible_actions(state)
        return [self.generate_edge(self, action) for action in actions]

    def has_unvisited_edge(self):
        return any([not edge.has_child() for edge in self.child_edges])

    def select_unvisited_edge(self):
        return [edge for edge in self.child_edges if not edge.has_child()][0]

    def visit_count(self):
        return sum([edge.visit_count for edge in self.child_edges])

class BaseEdge(object):

    def __init__(self, parent_node, action):
        self.action = action
        self.parent_node = parent_node
        self.child_node = None
        self.visit_count = 0

    def update_value(self, new_reward):
        err_msg = build_not_implemented_msg(self, "update_value")
        raise NotImplementedError(err_msg)

    def calculate_value(self):
        err_msg = build_not_implemented_msg(self, "calculate_value")
        raise NotImplementedError(err_msg)

    def has_child(self):
        return self.child_node is not None

    def build_child(self, state2node):
        child_state = self.parent_node.task.transit_state(self.parent_node.state, self.action)
        self.child_node = state2node(child_state)
        self.child_node.parent_edge = self

    def visit(self):
        self.visit_count += 1

class UCTNode(BaseNode):

    def generate_edge(self, parent_node, action):
        return UCTEdge(parent_node, action)

class UCTEdge(BaseEdge):

    def __init__(self, parent_node, action):
        super(UCTEdge, self).__init__(parent_node, action)
        self.C = 1.4142135623730951  # 1.41... = math.sqrt(2)
        self.value = 0

    def update_value(self, new_reward):
        self.value = self._calc_average_in_incremental_way(self.value, self.visit_count, new_reward)

    def calculate_value(self):
        if self.visit_count == 0:
            explore_term = float('inf')
        else:
            explore_term = math.sqrt(math.log(self.parent_node.visit_count()) / self.visit_count)
        return self.value + self.C * explore_term

    def _calc_average_in_incremental_way(self, old_value, visit_count, new_reward):
        assert visit_count != 0
        return old_value + 1.0 / visit_count * (new_reward - old_value)




import os
import sys
import StringIO

from tests.base_unittest import BaseUnitTest
from tests.utils import generate_tmp_dir_path, setup_tmp_dir, teardown_tmp_dir, remove_leaf_dir
from nose.tools import raises
from mock import patch, Mock
from kyoka.callback import BaseCallback, BasePerformanceWatcher, EpsilonAnnealer,\
        LearningRecorder, BaseFinishRule, ManualInterruption, WatchIterationCount
from kyoka.policy import EpsilonGreedyPolicy

def capture_log(instance):
    instance.capture = StringIO.StringIO()
    sys.stdout = instance.capture

def release_capture():
   sys.stdout = sys.__stdout__

class BaseCallbackTest(BaseUnitTest):

    def setUp(self):
        self.callback = BaseCallback()
        capture_log(self)

    def tearDown(self):
        release_capture()

    def test_interrupt_gpi(self):
        self.false(self.callback.interrupt_gpi("dummy", "dummy", "dummy"))

    def test_define_log_tag(self):
        self.eq("BaseCallback", self.callback.define_log_tag())

    def test_log(self):
        self.callback.log("hoge")
        self.eq("[BaseCallback] hoge\n", self.capture.getvalue())

class BasePerformanceWatcherTest(BaseUnitTest):

    def setUp(self):
        capture_log(self)

    def tearDown(self):
        release_capture()

    @raises(NotImplementedError)
    def test_implementation_check_on_define_performance_test_interval(self):
        BasePerformanceWatcher().define_performance_test_interval()

    @raises(NotImplementedError)
    def test_implementation_check_on_run_performance_test(self):
        BasePerformanceWatcher().run_performance_test("dummy", "dummy")

    def test_default_log_message(self):
        watcher = self.TestMinimumImplementation()
        watcher.before_gpi_start("dummy", "dummy")
        watcher.after_update(1, "dummy", "dummy")
        watcher.after_update(2, "dummy", "dummy")
        expected = "[TestMinimumImplementation] Performance test result : 3 (nb_iteration=2)\n"
        self.eq(expected, self.capture.getvalue())

    def test_setup_is_called(self):
        watcher = self.TestCompleteImplementation()
        self.false(watcher.is_setup_called)
        watcher.before_gpi_start("dummy", "dummy")
        self.true(watcher.is_setup_called)

    def test_teardown_is_called(self):
        watcher = self.TestCompleteImplementation()
        self.false(watcher.is_teardown_called)
        watcher.after_gpi_finish("dummy", "dummy")
        self.true(watcher.is_teardown_called)

    def test_timing_of_performance_test(self):
        watcher = self.TestCompleteImplementation()
        watcher.before_gpi_start("dummy", "dummy")
        watcher.after_update(1, "dummy", "dummy")
        self.eq('', self.capture.getvalue())
        watcher.after_update(2, "dummy", "dummy")
        self.eq('[Test] test:1\n', self.capture.getvalue())
        watcher.after_update(3, "dummy", "dummy")
        self.eq('[Test] test:1\n', self.capture.getvalue())
        watcher.after_update(4, "dummy", "dummy")
        self.eq('[Test] test:1\n[Test] test:4\n', self.capture.getvalue())
        self.eq([1, 4], watcher.performance_log)

    class TestCompleteImplementation(BasePerformanceWatcher):

        def __init__(self):
            self.is_setup_called = False
            self.is_teardown_called = False
            self.test_count = 0

        def setUp(self, task, value_function):
            self.is_setup_called = True

        def tearDown(self, task, value_function):
            self.is_teardown_called = True

        def define_performance_test_interval(self):
             return 2

        def run_performance_test(self, _task, _value_function):
            self.test_count += 1
            return self.test_count**2

        def define_log_message(self, iteration_count, task, value_function, test_result):
            return "test:%s" % test_result

        def define_log_tag(self):
            return "Test"

    class TestMinimumImplementation(BasePerformanceWatcher):

        def define_performance_test_interval(self):
            return 2

        def run_performance_test(self, _task, _value_function):
            return 3

class EpsilonAnnealerTest(BaseUnitTest):

    def setUp(self):
        self.policy = EpsilonGreedyPolicy()
        self.policy.set_eps_annealing(1.0, 0.1, 100)
        self.annealer = EpsilonAnnealer(self.policy)
        capture_log(self)

    def tearDown(self):
        release_capture()

    def test_define_log_tag(self):
        self.eq("EpsilonGreedyAnnealing", self.annealer.define_log_tag())

    def test_start_log(self):
        self.annealer.before_gpi_start("dummy", "dummy")
        self.include(str(1.0), self.capture.getvalue())
        self.include(str(0.1), self.capture.getvalue())

    def test_finish_log(self):
        [self.annealer.after_update(_, "dummy", "dummy") for _ in range(99)]
        self.not_include("finish", self.capture)
        self.annealer.after_update(100, "dummy", "dummy")
        self.include("finish", self.capture.getvalue())
        self.include(str(100), self.capture.getvalue())
        # Check that finish message is logged only once
        capture = StringIO.StringIO()
        sys.stdout = capture
        self.annealer.after_update(101, "dummy", "dummy")
        self.not_include("finish", capture.getvalue())

class LearningRecorderTest(BaseUnitTest):

    def setUp(self):
        self.algo = Mock()
        self.recorder = LearningRecorder(self.algo, generate_tmp_dir_path(__file__), 2)
        capture_log(self)

    def tearDown(self):
        release_capture()
        gen_dpath = lambda fname: os.path.join(generate_tmp_dir_path(__file__), fname)
        for idx in range(1, 5):
            dname = "after_%d_iteration" % idx
            remove_leaf_dir(gen_dpath(dname), [])
        remove_leaf_dir(gen_dpath("gpi_finished"), [])
        teardown_tmp_dir(__file__, [])

    def test_start_log(self):
        setup_tmp_dir(__file__)
        self.recorder.before_gpi_start("dummy", "dummy")
        self.include(str(2), self.capture.getvalue())
        self.include(generate_tmp_dir_path(__file__), self.capture.getvalue())

    @raises(Exception)
    def test_before_gpi_start(self):
        self.recorder.before_gpi_start("dummy", "dummy")

    def test_after_update(self):
        setup_tmp_dir(__file__)
        gen_dpath = lambda fname: os.path.join(generate_tmp_dir_path(__file__), fname)
        self.recorder.after_update(1, "dummy", "dummy")
        self.algo.save.assert_not_called()
        self.false(os.path.exists(gen_dpath("after_1_iteration")))
        self.recorder.after_update(2, "dummy", "dummy")
        self.true(os.path.exists(gen_dpath("after_2_iteration")))
        self.algo.save.assert_called_with(gen_dpath("after_2_iteration"))
        self.recorder.after_update(3, "dummy", "dummy")
        self.false(os.path.exists(gen_dpath("after_3_iteration")))
        self.algo.save.assert_called_with(gen_dpath("after_2_iteration"))
        self.recorder.after_update(4, "dummy", "dummy")
        self.true(os.path.exists(gen_dpath("after_4_iteration")))
        self.algo.save.assert_called_with(gen_dpath("after_4_iteration"))
        self.include(gen_dpath("after_4_iteration"), self.capture.getvalue())

    def test_after_gpi_finish(self):
        setup_tmp_dir(__file__)
        gen_dpath = lambda fname: os.path.join(generate_tmp_dir_path(__file__), fname)
        gen_dpath = lambda fname: os.path.join(generate_tmp_dir_path(__file__), fname)
        self.false(os.path.exists(gen_dpath("gpi_finished")))
        self.recorder.after_gpi_finish("dummy", "dummy")
        self.true(os.path.exists(gen_dpath("gpi_finished")))
        self.algo.save.assert_called_with(gen_dpath("gpi_finished"))

class BaseFinishRuleTest(BaseUnitTest):

    def setUp(self):
        self.rule = BaseFinishRule()
        capture_log(self)

    def tearDown(self):
        release_capture()

    @raises(NotImplementedError)
    def test_check_condition(self):
        self.rule.check_condition("dummy", "dummy", "dummy")

    @raises(NotImplementedError)
    def test_generate_start_message(self):
        self.rule.generate_start_message()

    @raises(NotImplementedError)
    def test_generate_finish_message(self):
        self.rule.generate_finish_message("dummy")

    def test_interrupt_gpi(self):
        rule = self.TestImplementation()
        self.false(rule.interrupt_gpi(1, "dummy", "dummy"))
        self.true(rule.interrupt_gpi(2, "dummy", "dummy"))
        self.false(rule.interrupt_gpi(3, "dummy", "dummy"))

    def test_log_start_message(self):
        rule = self.TestImplementation()
        rule.before_gpi_start("dummy", "dummy")
        self.eq('[test_tag] start\n', self.capture.getvalue())

    def test_log_finish_message(self):
        rule = self.TestImplementation()
        self.false(rule.interrupt_gpi(1, "task", "value_function"))
        self.eq('', self.capture.getvalue())
        self.true(rule.interrupt_gpi(2, "task", "value_function"))
        self.eq('[test_tag] finish:2\n', self.capture.getvalue())

    class TestImplementation(BaseFinishRule):

        def define_log_tag(self):
            return "test_tag"

        def check_condition(self, iteration_count, _task, _value_function):
            return iteration_count == 2

        def generate_start_message(self):
            return "start"

        def generate_finish_message(self, iteration_count):
            return "%s:%s" % ("finish", iteration_count)

class ManualInterruptionTest(BaseUnitTest):

    def setUp(self):
        file_path = self.__generate_tmp_file_path()
        self.rule = ManualInterruption(monitor_file_path=file_path, watch_interval=5)

    def tearDown(self):
        file_path = self.__generate_tmp_file_path()
        if os.path.isfile(file_path): os.remove(file_path)

    def test_define_log_tag(self):
        self.eq("ManualInterruption", self.rule.define_log_tag())

    def test_check_condition(self):
        file_path = self.__generate_tmp_file_path()
        mock_return = [1, 2, 3, 6, 7, 12]
        with patch('time.time', side_effect=mock_return):
            self.rule.generate_start_message()
            self.false(self.rule.check_condition("dummy", "dummy", "dummy"))
            self.__write_word(file_path, "hoge")
            self.false(self.rule.check_condition("dummy", "dummy", "dummy"))
            self.false(self.rule.check_condition("dummy", "dummy", "dummy"))
            self.__write_word(file_path, "stop")
            self.false(self.rule.check_condition("dummy", "dummy", "dummy"))
            self.true(self.rule.check_condition("dummy", "dummy", "dummy"))

    def test_generate_start_message(self):
        message = self.rule.generate_start_message()
        self.include(self.rule.TARGET_WARD, message)
        self.include(self.rule.monitor_file_path, message)
        self.include(str(self.rule.watch_interval), message)

    def test_generate_finish_message(self):
        file_path = self.__generate_tmp_file_path()
        msg = self.rule.generate_finish_message(5)
        self.include(str(5), msg)
        self.include(file_path, msg)

    def __generate_tmp_file_path(self):
        return os.path.join(os.path.dirname(__file__), "tmp_file_for_manual_interruption_test.tmp")

    def __write_word(self, filepath, word):
      with open(filepath, 'wb') as f: f.write(word)

class WatchIterationCountTest(BaseUnitTest):

    def setUp(self):
        self.rule = WatchIterationCount(target_count=100)

    def tearDown(self):
        release_capture()

    def test_define_log_tag(self):
        self.eq("Progress", self.rule.define_log_tag())

    def test_check_condition(self):
        self.false(self.rule.check_condition(99, "dummy", "dummy"))
        self.true(self.rule.check_condition(100, "dummy", "dummy"))
        self.true(self.rule.check_condition(101, "dummy", "dummy"))

    def test_generate_start_message(self):
        self.include(str(100), self.rule.generate_start_message())

    def test_generate_finish_message(self):
        with patch('time.time', side_effect=[1477394336.834469, 1477394338.815435]):
            self.rule.generate_start_message()
            message = self.rule.generate_finish_message(5)
            self.include(str(5), message)
            self.include(str(1), message)

    def test_message_for_lifecycle(self):
        rule = WatchIterationCount(target_count=2)
        mock_time_return = [
            1477462649.467603,
            1477462653.996855,
            1477462655.032283,
            1477462658.945142,
            1477462663.027276,
            1477462679.056145
        ]
        expected_calc_time = [1.0354280471801758, 4.082134008407593]
        expected_interrupt_return = [False, True]
        expected_total_time = 29.588541984558105

        with patch('time.time', side_effect=mock_time_return):
            rule.before_gpi_start("dummy", "dummy")
            for idx, expected in enumerate(expected_calc_time, start=1):
                capture = StringIO.StringIO()
                sys.stdout = capture
                rule.before_update("dummy", "dummy", "dummy")
                rule.after_update(idx, "dummy", "dummy")
                self.eq(expected_interrupt_return[idx-1], rule.interrupt_gpi(idx, "dummy", "dummy"))
                self.include("[Progress]", capture.getvalue())
                self.include("%.1f" % expected, capture.getvalue())
                self.include("%d /" % idx, capture.getvalue())
            rule.after_gpi_finish("dummy", "dummy")
            self.include("%d" % expected_total_time, capture.getvalue())

    def test_verbose_mode(self):
        capture = StringIO.StringIO()
        sys.stdout = capture
        rule = WatchIterationCount(target_count=100, verbose=0)
        rule.before_gpi_start("dummy", "dummy")
        rule.before_update(1, "dummy", "dummy")
        rule.after_update(1, "dummy", "dummy")
        rule.interrupt_gpi(1, "dummy", "dummy")
        rule.before_update(100, "dummy", "dummy")
        rule.after_update(100, "dummy", "dummy")
        rule.interrupt_gpi(100, "dummy", "dummy")
        rule.after_gpi_finish("dummy", "dummy")
        self.include("Completed", capture.getvalue())
        self.not_include("Finished", capture.getvalue())




import pickle

def build_not_implemented_msg(instance, method_name):
    base_msg = "[ {0} ] class does not implement [ {1} ] method"
    return base_msg.format(instance.__class__.__name__, method_name)

def pickle_data(file_path, data):
    with open(file_path, "wb") as f: pickle.dump(data, f)

def unpickle_data(file_path):
    with open(file_path, "rb") as f: return pickle.load(f)

def value_function_check(algorithm_name, valid_types, value_function):
    if not any([isinstance(value_function, valid_type) for valid_type in valid_types]):
        base_err_msg = 'You passed invalid type of value function for "%s" algorithm. '+\
                '(value function for "%s" algorithm must be child class of [%s])'
        valid_type_names = " or ".join([v_type.__name__ for v_type in valid_types])
        raise TypeError(base_err_msg % (algorithm_name, algorithm_name, valid_type_names))

